(function() {
	
	var addEnemies = function(game, count) {
		var enemy = null;
		var id = 0;
		
		for (var i = 0; i < count; i++) {
			// get random id from 1 to 5
			id = (Math.random() * 4 + 0.5 | 0) + 1;
			new Enemy(game, id);
		}
	};
	
	var playEffectAnimation = function(effect) {
		// dealy animation so we don't start same effects at same frame
		window.setTimeout(function() {
			// default animation is added by parser while creating objects
			effect.animations.play("default", null, true);
		}, Math.random() * 1000);
	};
	
	// time after time changes ship direction.. so it doesn't float away
	var changeShipDirection = function(ship) {
		ship.body.velocity.x *= -1;
		if (ship.body.velocity.x < 0) {
			ship.scale.x = -1;
		} 
		else {
			ship.scale.x = 1;
		}
		window.setTimeout(function() {
			// check if ship is still alive
			if (ship.parent) {
				changeShipDirection(ship);
			}
		}, 5000 + 5000 * Math.random());
	};
	
	var Gun = function(gunGroup) {
		this.sprite = gunGroup.gun;
		this.sprite.bringToTop();
		
		this.shootSprite = gunGroup.gun_shoot;
		// hide shooting sprite
		this.shootSprite.kill();
		
		this.shootAnimation = this.shootSprite.animations.add("shoot");
		
		
		var that = this;
		this.shootAnimation.onComplete.add(function() {
			that.isShooting = false;
		});
	
	};
	Gun.prototype = {
		isShooting: false,	
		shoot: function() {
			if (this.isShooting) {
				return;
			}
			this.isShooting = true;
			this.shootSprite.revive();
			
			this.shootAnimation.play(10, false, true);
			// TODO: add sound effect here
			
		}
	
	};
	
	var Player = function(playerGroup) {
		this.group = playerGroup;
		
		// default animation is added by parser while creating objects
		this.group.character.animations.play("default", 10, true);
		
		this.arms = {
			left: this.group.LeftArm.self,
			right: this.group.RightArm.self
		};
		
		this.guns = {
			left: new Gun(this.group.LeftArm),
			right: new Gun(this.group.RightArm)
		}
	};
	
	Player.prototype = {
		turnArm: function(arm, y, x) {
			arm.rotation = Math.atan2(y, x);
		}
	};
	
	
	
	var Enemy = function(game, id) {
		var that = this;
		
		this.direction = game.math.randomSign();
		
		this.sprite = game.add.sprite(this.direction * 500, 0, "/entities/enemy0" + id + ".png");
		
		
		var framesTotal = game.cache.getFrameData("/entities/enemy0" + id + ".png").total;
		
		var animation = this.sprite.animations.add("run");

		// flip sprite and animation frames
		if (this.direction < 0) {
			this.sprite.scale.x = this.direction;
			animation._frames.reverse();
		}
		
		animation.play(10, true);
		
		
		
		
		this.sprite.inputEnabled = true;
		this.sprite.events.onInputUp.add(function() {
			that.kill();
		});
		
		
		game.physics.arcade.enable(this.sprite);
		// place sprite on the ground; bottom.y = 0
		this.sprite.body.y = -this.sprite.height;

		// start running, but delay start as we don't want that all enemies will go in one big chunk
		this.runDelayed(Math.random() * 5000);
		
		
		this.thombstone = game.add.sprite(0, 0, "thombstone");
		this.tsAnim = this.thombstone.animations.add("show");
		this.thombstone.anchor.y = 1;

		// thombstone has an appearing animation and we should move it slighly lower
		this.thombstone.y = 8;

		// hide thombstone
		this.thombstone.kill();
		
		this.thombstone.inputEnabled = true;
		this.thombstone.events.onInputUp.add(function() {
			// reverse animation
			that.tsAnim._frames.reverse();
			that.tsAnim.onComplete.addOnce(function() {
				that.tsAnim._frames.reverse();
			});
			that.tsAnim.play(50, false, true);
		});
	
	};
	
	Enemy.prototype = {
		kill: function() {
			// TODO: add fancy dying animation and bonus coins
			this.thombstone.revive();
			this.thombstone.bringToTop();

			// stop previous animation and make sure that frames are in correct order
			this.tsAnim.stop();
			this.tsAnim._frames.sort();
			
			this.tsAnim.play(10, false, false);
			this.thombstone.x = this.sprite.x + (this.sprite.width - this.thombstone.width) * 0.5;
			
			this.sprite.kill();
			
			// start running again after a while
			this.runDelayed(Math.random() * 5000);
		},
		
		runDelayed: function(delay) {
			var that = this;
			window.setTimeout(function() {
				that.run();
			}, delay);
		},
		
		run: function() {
			this.sprite.revive();
			this.sprite.body.x = this.direction * 500;
			this.sprite.body.velocity.x = -100 * this.direction;
		
		}
	};

	// create phaser game
	window.game = new Phaser.Game(800, 600, Phaser.AUTO, '', {
		preload: function() {
			// load saved assets
			mt.preload(this.game);
		},
		create: function() {
			var that = this;

			// set target cursor
			var cursor = mt.assets.character["cursor.png"];
			document.body.style.cursor = "url(" + mt.getAssetPath(cursor) + ") " + (cursor.width * 0.5) + " " + cursor.height * 0.5 + ", default";

			// recreate saved map
			mt.create();

			// add basic physics
			this.game.physics.startSystem(Phaser.Physics.ARCADE);


			// as we have created map with center point 0,0
			// we need to adjust camera bounds and position
			this.game.camera.bounds.x = this.game.camera.x = -400;
			this.game.camera.bounds.y = this.game.camera.y = -480;

			// create Player character
			var player = new Player(mt.objects.Character);

			// add some enemies
			addEnemies(this.game, 10);

			//play effects animations if sprite has animation
			var effects = mt.objects.Effects;
			for (var i in effects) {
				if (!effects[i].animations) {
					continue;
				}
				playEffectAnimation(effects[i]);
			}
			
			// enable basic arcade physics
			game.physics.arcade.enable(effects.ship);
			
			// start moving around ship
			effects.ship.body.velocity.x = -10;
			changeShipDirection(effects.ship);


			// add event callbacks
			// make arms look at cursor and shoot on mouse down
			this.game.input.onDown.add(function(pointer) {
				var x = pointer.x + pointer.game.camera.x;
				var y = pointer.y + pointer.game.camera.y;
				if (x < 0) {
					player.turnArm(player.arms.left, player.arms.left.y - y, player.arms.left.x - x);
					player.guns.left.shoot();
				} 
				else {
					player.turnArm(player.arms.right, y - player.arms.right.y, x - player.arms.right.x);
					player.guns.right.shoot();
				}
			});
			
			this.game.input.moveCallback = function(pointer) {
				var x = pointer.x + pointer.game.camera.x;
				var y = pointer.y + pointer.game.camera.y;
				
				if (x < 0) {
					player.turnArm(player.arms.left, player.arms.left.y - y, player.arms.left.x - x);
				} 
				else {
					player.turnArm(player.arms.right, y - player.arms.right.y, x - player.arms.right.x);
				}
			
			};
		}
	});

})();
